%global _hardened_build 1
%global systemctl_bin /usr/bin/systemctl
%global selinuxtype targeted
%global modulename slapd
%global with_selinux 0
%if 0%{?centos_ver} && 0%{?centos_ver} == 8
%global with_selinux 1
%endif
%global _olversion 2.5.8
%global _olrelease 1%{?dist}

%define _olprefix /opt/symas
%define _varfix /var/symas
%define _ollibdir %{_olprefix}/lib
%define _etcfix %{_olprefix}/etc
%define _sedatadir /usr/share
%define _sesharedstatedir /usr/com


Name:		symas-openldap
Version:	%{_olversion}
Release:	%{_olrelease}
Summary:	Symas's openldap build

License:	BSD
URL:		https://www.openldap.org

BuildRequires:	symas-openssl-devel, symas-cyrus-sasl-devel
BuildRequires:	gcc, libtool-ltdl-devel, symas-heimdal-devel
BuildRequires:	symas-libevent-devel, symas-libargon2-devel
BuildRequires:	cracklib-devel, systemd-devel
Source0: ftp://ftp.OpenLDAP.org/pub/OpenLDAP/openldap-release/openldap-%{_olversion}.tgz
Source1: symas-openldap-servers.service
Source2: all_threads_full_bt.gdb
Source3: core_grab.gdb
Source4: core_wait.gdb
Source5: default_attach.gdb
Source6: connexitor.ldif
Source7: connexitor.schema
Source8: edupermission.ldif
Source9: edupermission.schema
Source10: eduperson.ldif
Source11: eduperson.schema
Source12: krb5-kdc.ldif
Source13: krb5-kdc.schema
Source14: qmail.ldif
Source15: qmail.schema
Source16: rfc2307bis.ldif
Source17: rfc2307bis.schema
Source18: samba.ldif
Source19: samba.schema
Source20: samba3.ldif
Source21: samba3.schema
Source22: %{modulename}.te
Source23: %{modulename}.if
Source24: %{modulename}.fc
Source25: exampledb.sh
Source26: exampledb-otp.sh
Source27: 50-soldap.sh
Source28: 50-soldap.csh

AutoReqProv: No

Patch0: Install-slapd-watcher.patch
Patch1: passwd.patch
Patch2: ITS9616.patch

%description
Symas's OpenLDAP 2.5 build

%package libs
Summary:	openldap Libaries
Requires:	symas-openssl-libs, symas-cyrus-sasl-libs
AutoReqProv:	no

%description libs
The symas-openldap-libs package contains the openldap libraries

%package devel
Summary:	openldap Development
Requires:	symas-openldap-libs = %{_olversion}-%{_olrelease}
AutoReqProv:	no

%description devel
The symas-openldap-devel package contains the linking libraries and include files

%package servers
Summary:	openldap server binaries
Requires:	symas-openldap-libs = %{_olversion}-%{_olrelease}, symas-libargon2-libs, symas-heimdal-libs
%if 0%{?centos_ver} && 0%{?centos_ver} == 7
Requires:	cracklib >= 2.9.0
%else
Requires:	cracklib >= 2.9.6
%endif
Requires:	symas-libevent-libs
%if 0%{?suse_version}
Requires:	libltdl7
%else
Requires:	libtool-ltdl
%endif
%if 0%{?with_selinux}
# This ensures that the *-selinux package and all it’s dependencies are not pulled
# into containers and other systems that do not use SELinux
Requires:	(symas-openldap-servers-selinux if selinux-policy-%{selinuxtype})
%endif
Provides:	openldap-servers
Conflicts:	openldap-servers
AutoReqProv:	no

%description servers
The symas-openldap-servers package contains slapd and its modules

%if 0%{?with_selinux}
# SELinux subpackage
%package servers-selinux
Summary: SELinux policies for symas-openldap-servers
BuildArch: noarch
Version: 1.0.2
Release: 1%{?dist}
License: GPLv2
Requires: selinux-policy-%{selinuxtype}
Requires(post): selinux-policy-%{selinuxtype}
BuildRequires: selinux-policy-devel
%{?selinux_requires}

%description servers-selinux
SELinux policy modules for symas-openldap-servers
%endif

%package clients
Summary:	openldap client binaries
Requires: symas-openldap-libs = %{_olversion}-%{_olrelease}
AutoReqProv:	no

%description clients
The symas-openldap-clients package contains client tools such as ldapsearch

%package moddev
Summary: OpenLDAP module development pieces
Requires: symas-openldap-devel = %{_olversion}-%{_olrelease}
AutoReqProv:	no

%description moddev
This package allows development of slapd overlays and modules using the
OpenlDAP libraries. It includes module specific headers and the libtool
utility required for compilation.

%prep
%setup -qn openldap-%{_olversion}
%patch0 -p1
%patch1 -p1
%patch2 -p1

%build
%if 0%{?suse_version}
export LDFLAGS="-L%{_ollibdir} -Wl,-rpath,%{_ollibdir}"
%else
export LDFLAGS="%{__global_ldflags} -L%{_ollibdir} -Wl,-rpath,%{_ollibdir}"
%endif
export CFLAGS="%{optflags} -fPIC"
export CPPFLAGS=-I%{_olprefix}/include
./configure --prefix=%{_olprefix} \
    --with-cyrus-sasl \
    --with-tls=openssl \
    --libexecdir=%{_ollibdir} \
    --datadir=%{_varfix} \
    --sysconfdir=%{_etcfix} \
    --sharedstatedir=%{_etcfix} \
    --localstatedir=%{_varfix} \
    --enable-dynamic \
    --enable-slapd \
    --enable-modules \
    --enable-balancer=mod \
    --enable-backends=mod \
		--disable-sql \
		--disable-ndb \
		--disable-perl \
		--disable-wt \
    --enable-overlays=mod \
		--disable-seqmod \
    --enable-argon2 \
    --enable-debug \
    --enable-spasswd \
    --enable-crypt
make depend
make DEFINES="-DSLAP_SCHEMA_EXPOSE"
make -C contrib/slapd-modules/autogroup prefix=%{_olprefix} libexecdir=%{_ollibdir}
make -C contrib/slapd-modules/datamorph prefix=%{_olprefix} libexecdir=%{_ollibdir}
make -C contrib/slapd-modules/lastbind prefix=%{_olprefix} libexecdir=%{_ollibdir}
make -C contrib/slapd-modules/noopsrch prefix=%{_olprefix} libexecdir=%{_ollibdir}
make -C contrib/slapd-modules/passwd/pbkdf2 prefix=%{_olprefix} libexecdir=%{_ollibdir}
make -C contrib/slapd-modules/passwd/sha2 prefix=%{_olprefix} libexecdir=%{_ollibdir}
make -C contrib/slapd-modules/ppm prefix=%{_olprefix} libexecdir=%{_ollibdir} etcdir=%{_etcfix} OPT=-fPIC
make -C contrib/slapd-modules/rbac prefix=%{_olprefix} libexecdir=%{_ollibdir}
make -C contrib/slapd-modules/smbk5pwd prefix=%{_olprefix} libexecdir=%{_ollibdir} \
 HEIMDAL_INC='-I%{_olprefix}/include/heimdal' HEIMDAL_LIB='-L%{_ollibdir} -lkrb5 -lkadm5srv' \
 SSL_INC='-I/opt/symas/include' SSL_LIB='-L/opt/symas/lib -lcrypto'
make -C contrib/slapd-modules/variant prefix=%{_olprefix} libexecdir=%{_ollibdir}
pushd libraries/liblmdb
export XCFLAGS="%{optflags}"
make prefix=%{_olprefix}
popd
%if 0%{?with_selinux}
# SELinux policy (originally from selinux-policy-contrib)
# this policy module will override the production module
mkdir selinux
cp -p %{SOURCE22} %{SOURCE23} %{SOURCE24} selinux/
cd selinux
make -f %{_sedatadir}/selinux/devel/Makefile %{modulename}.pp
bzip2 -9 %{modulename}.pp
%endif

%install
make install DESTDIR=%{buildroot} STRIP_OPTS=""
make -C contrib/slapd-modules/autogroup install prefix=%{_olprefix} libexecdir=%{_ollibdir} DESTDIR=%{buildroot} STRIP_OPTS=""
make -C contrib/slapd-modules/datamorph install prefix=%{_olprefix} libexecdir=%{_ollibdir} DESTDIR=%{buildroot} STRIP_OPTS=""
make -C contrib/slapd-modules/lastbind install prefix=%{_olprefix} libexecdir=%{_ollibdir} DESTDIR=%{buildroot} STRIP_OPTS=""
make -C contrib/slapd-modules/noopsrch install prefix=%{_olprefix} libexecdir=%{_ollibdir} DESTDIR=%{buildroot} STRIP_OPTS=""
make -C contrib/slapd-modules/passwd/pbkdf2 install prefix=%{_olprefix} libexecdir=%{_ollibdir} DESTDIR=%{buildroot} STRIP_OPTS=""
make -C contrib/slapd-modules/passwd/sha2 install prefix=%{_olprefix} libexecdir=%{_ollibdir} DESTDIR=%{buildroot} STRIP_OPTS=""
make -C contrib/slapd-modules/ppm install prefix=%{_olprefix} libexecdir=%{_ollibdir} DESTDIR=%{buildroot} STRIP_OPTS="" etcdir=%{_etcfix}
make -C contrib/slapd-modules/rbac install prefix=%{_olprefix} libexecdir=%{_ollibdir} DESTDIR=%{buildroot} STRIP_OPTS=""
make -C contrib/slapd-modules/smbk5pwd install prefix=%{_olprefix} libexecdir=%{_ollibdir} DESTDIR=%{buildroot} STRIP_OPTS=""
make -C contrib/slapd-modules/variant install prefix=%{_olprefix} libexecdir=%{_ollibdir} DESTDIR=%{buildroot} STRIP_OPTS=""

pushd libraries/liblmdb
make install prefix=%{_olprefix} DESTDIR=%{buildroot}
popd

pushd tests/progs
make install prefix=%{_olprefix} DESTDIR=%{buildroot}
popd

rm -f %{buildroot}%{_olprefix}/include/lmdb.h
rm -f %{buildroot}%{_ollibdir}/liblmdb.a
rm -f %{buildroot}%{_ollibdir}/liblmdb.so
rm -f %{buildroot}%{_ollibdir}/openldap/autogroup.a
rm -f %{buildroot}%{_ollibdir}/openldap/datamorph.a
rm -f %{buildroot}%{_ollibdir}/openldap/lastbind.a
rm -f %{buildroot}%{_ollibdir}/openldap/noopsrch.a
rm -f %{buildroot}%{_ollibdir}/openldap/ppm.a
rm -f %{buildroot}%{_ollibdir}/openldap/pw-pbkdf2.a
rm -f %{buildroot}%{_ollibdir}/openldap/pw-sha2.a
rm -f %{buildroot}%{_ollibdir}/openldap/rbac.a
rm -f %{buildroot}%{_ollibdir}/openldap/smbk5pwd.a
rm -f %{buildroot}%{_ollibdir}/openldap/variant.a
rm -f %{buildroot}%{_etcfix}/openldap/ppm.example
rm -f %{buildroot}%{_ollibdir}/*.la
rm -f %{buildroot}%{_unitdir}/slapd.service
mkdir -p %{buildroot}%{_varfix}/openldap-data
mkdir -p %{buildroot}%{_varfix}/run
chmod 0755 %{buildroot}%{_ollibdir}/lib*.so*
chmod 0644 %{buildroot}%{_ollibdir}/openldap/*.la
install -m 0700 -d %{buildroot}%{_varfix}/openldap-data
install -m 0755 -d %{buildroot}%{_varfix}/run

# install an init script for the servers
mkdir -p %{buildroot}%{_unitdir}
install -m 0644 %SOURCE1 %{buildroot}%{_unitdir}/symas-openldap-servers.service
pushd %{buildroot}%{_unitdir}
ln -s symas-openldap-servers.service slapd.service
popd

mkdir -p %{buildroot}%{_olprefix}/bin/moddev
cp libtool %{buildroot}%{_olprefix}/bin/moddev/
mkdir -p %{buildroot}%{_olprefix}/include/moddev/ac
cp include/lber_pvt.h %{buildroot}%{_olprefix}/include/moddev/
cp include/ldap_cdefs.h %{buildroot}%{_olprefix}/include/moddev/
cp include/ldap_features.h %{buildroot}%{_olprefix}/include/moddev/
cp include/lutil.h %{buildroot}%{_olprefix}/include/moddev/
cp include/portable.h %{buildroot}%{_olprefix}/include/moddev/
cp include/ac/assert.h %{buildroot}%{_olprefix}/include/moddev/ac/
cp include/ac/fdset.h %{buildroot}%{_olprefix}/include/moddev/ac/
cp include/ac/localize.h %{buildroot}%{_olprefix}/include/moddev/ac/
cp include/ac/socket.h %{buildroot}%{_olprefix}/include/moddev/ac/
cp include/ac/string.h %{buildroot}%{_olprefix}/include/moddev/ac/

mkdir -p %{buildroot}%{_ollibdir}/gdb
install -m 0644 %SOURCE2 %{buildroot}%{_ollibdir}/gdb/
install -m 0644 %SOURCE3 %{buildroot}%{_ollibdir}/gdb/
install -m 0644 %SOURCE4 %{buildroot}%{_ollibdir}/gdb/
install -m 0644 %SOURCE5 %{buildroot}%{_ollibdir}/gdb/

install -m 0644 %SOURCE6 %{buildroot}%{_etcfix}/openldap/schema/
install -m 0644 %SOURCE7 %{buildroot}%{_etcfix}/openldap/schema/
install -m 0644 %SOURCE8 %{buildroot}%{_etcfix}/openldap/schema/
install -m 0644 %SOURCE9 %{buildroot}%{_etcfix}/openldap/schema/
install -m 0644 %SOURCE10 %{buildroot}%{_etcfix}/openldap/schema/
install -m 0644 %SOURCE11 %{buildroot}%{_etcfix}/openldap/schema/
install -m 0644 %SOURCE12 %{buildroot}%{_etcfix}/openldap/schema/
install -m 0644 %SOURCE13 %{buildroot}%{_etcfix}/openldap/schema/
install -m 0644 %SOURCE14 %{buildroot}%{_etcfix}/openldap/schema/
install -m 0644 %SOURCE15 %{buildroot}%{_etcfix}/openldap/schema/
install -m 0644 %SOURCE16 %{buildroot}%{_etcfix}/openldap/schema/
install -m 0644 %SOURCE17 %{buildroot}%{_etcfix}/openldap/schema/
install -m 0644 %SOURCE18 %{buildroot}%{_etcfix}/openldap/schema/
install -m 0644 %SOURCE19 %{buildroot}%{_etcfix}/openldap/schema/
install -m 0644 %SOURCE20 %{buildroot}%{_etcfix}/openldap/schema/
install -m 0644 %SOURCE21 %{buildroot}%{_etcfix}/openldap/schema/

mkdir -p %{buildroot}%{_olprefix}/share/symas
install -m 0755 %SOURCE25 %{buildroot}%{_olprefix}/share/symas
install -m 0755 %SOURCE26 %{buildroot}%{_olprefix}/share/symas

mkdir -p %{buildroot}/etc/profile.d
install -m 0644 %SOURCE27 %{buildroot}/etc/profile.d
install -m 0644 %SOURCE28 %{buildroot}/etc/profile.d

%if 0%{?with_selinux}
install -D -m 0644 selinux/%{modulename}.pp.bz2 %{buildroot}%{_sedatadir}/selinux/packages/%{selinuxtype}/%{modulename}.pp.bz2
%endif

%check
make test

%if 0%{?with_selinux}
# SELinux contexts are saved so that only affected files can be
# relabeled after the policy module installation
%pre servers-selinux
%selinux_relabel_pre -s %{selinuxtype}

%post servers-selinux
%selinux_modules_install -s %{selinuxtype} %{_sedatadir}/selinux/packages/%{selinuxtype}/%{modulename}.pp.bz2
%selinux_relabel_post -s %{selinuxtype}

if [ "$1" -le "1" ]; then # First install
   # the daemon needs to be restarted for the custom label to be applied
   %systemd_postun_with_restart %{modulename}.service
fi

%postun servers-selinux
if [ $1 -eq 0 ]; then
    %selinux_modules_uninstall -s %{selinuxtype} %{modulename}
	%selinux_relabel_post -s %{selinuxtype}
fi

# if with_selinux
%endif

%post servers
%systemd_post symas-openldap-servers.service

%preun servers
%systemd_preun symas-openldap-servers.service

%postun servers
%systemd_postun_with_restart symas-openldap-servers.service


%files libs
%defattr(-,root,root)
%{_etcfix}/openldap/ldap.conf.default
%{_ollibdir}/*.so.*
%{_olprefix}/share/man/man5/ldap.conf.5
%{_olprefix}/share/man/man5/ldif.5
/etc/profile.d/50-soldap*

%files devel
%defattr(-,root,root)
%{_olprefix}/include/*.h
%{_ollibdir}/*.a
%{_ollibdir}/*.so
%{_ollibdir}/pkgconfig/*.pc
%{_olprefix}/share/man/man3

%files servers
%defattr(-,root,root)
%{_olprefix}/bin/mdb_*
%{_olprefix}/sbin
%{_etcfix}
%{_ollibdir}/slapd
%{_ollibdir}/openldap
%{_ollibdir}/gdb
%{_olprefix}/share/man/man1/mdb_*
%{_olprefix}/share/man/man5/slap*.5
%{_olprefix}/share/man/man5/lload*.5
%{_olprefix}/share/man/man8
%{_olprefix}/share/symas
%{_varfix}
%{_unitdir}/symas-openldap-servers.service
%{_unitdir}/slapd.service
%exclude %{_etcfix}/openldap/ldap.conf
%exclude %{_etcfix}/openldap/ldap.conf.default
%exclude %{_etcfix}/openldap/slapd.conf
%exclude %{_etcfix}/openldap/slapd.ldif
%exclude %{_olprefix}/share/man/man5/ldap.conf.5
%exclude %{_olprefix}/share/man/man5/ldif.5

%if 0%{?with_selinux}
%files servers-selinux
%{_sedatadir}/selinux/packages/%{selinuxtype}/%{modulename}.pp.*
%ghost %verify(not md5 size mtime) %{_sesharedstatedir}/selinux/%{selinuxtype}/active/modules/200/%{modulename}
%endif

%files clients
%defattr(-,root,root)
%{_olprefix}/bin
%{_olprefix}/share/man/man1
%exclude %{_olprefix}/bin/mdb_*
%exclude %{_olprefix}/share/man/man1/mdb_*
%exclude %{_olprefix}/bin/moddev

%files moddev
%{_olprefix}/include/moddev
%{_olprefix}/bin/moddev/libtool

%changelog
* Mon Oct 11 2021 Symas Support <support@symas.com> - 2.5.8-1
- OpenLDAP 2.5.8 release
* Thu Oct 07 2021 Symas Support <support@symas.com> - 2.5.7-3
- Add patch for ITS9712
* Wed Sep 15 2021 Symas Support <support@symas.com> - 2.5.7-2
- Fix memory leak in sessionlog parsing (ITS#6467)
- Fix memory leak in LDAP_OPT_X_TLS_PEERCERT (ITS#9696)
* Thu Aug 19 2021 Symas Support <support@symas.com> - 2.5.7-1
- OpenLDAP 2.5.7 release
* Wed Jul 28 2021 Symas Support <support@symas.com> - 2.5.6-1
- OpenLDAP 2.5.6 release
- ITS9616 patch
- Add example database scripts
- Add profile.d settings to add Symas paths to shells
* Tue Jul 13 2021 Symas Support <support@symas.com> - 2.5.5-2
- Fix systemd unit script alias to be a softlink
- ITS9578.patch
- ITS8747.patch
- ITS9363.patch
- ITS9590.patch
- ITS9601.patch
- ITS9602.patch
- ITS9603.patch
- ITS9608.patch
- ITS7832.patch
* Tue Jun 08 2021 Symas Support <support@symas.com> - 2.5.5-1
- OpenLDAP 2.5.5 release
* Tue May 25 2021 Symas Support <support@symas.com> - 2.5.4-5
- Change paths to match odd Symas Gold locations
* Tue May 25 2021 Symas Support <support@symas.com> - 2.5.4-4
- ITS6467-4.patch
- ITS6467-5.patch
* Thu May 13 2021 Symas Support <support@symas.com> - 2.5.4-3
- ITS9534.patch
- ITS9551.patch
- ITS6467-2.patch
- ITS9552.patch
- ITS6467-3.patch
* Tue May 11 2021 Symas Support <support@symas.com> - 2.5.4-2
- ITS6457 patch
- ITS6207 patch
- ITS6467.patch
- ITS8721 patch
- ITS8747 patch
- ITS9502 patch
- ITS9529 patch
- ITS9530 patch
- ITS9531 patch
- ITS9537 patch
- ITS9541 patch
- ITS9542 patch
- ITS9544 patch
- ITS9546 patch
- ITS9548 patch
* Tue Apr 6 2021 Symas Support <support@symas.com> - 2.5.4-1
- initial packaging
